/*
 * Copyright (c) 2010, 2011, Oracle and/or its affiliates. All rights reserved.
 *
 * This software and related documentation are provided under a license 
 * agreement containing restrictions on use and disclosure and are protected 
 * by intellectual property laws. Except as expressly permitted in your 
 * license agreement or allowed by law, you may not use, copy, reproduce, 
 * translate, broadcast, modify, license, transmit, distribute,
 * exhibit, perform, publish, or display any part, in any form, or by any 
 * means. Reverse engineering, disassembly, or decompilation of this software,
 * unless required by law for interoperability, is prohibited.
 *
 * The information contained herein is subject to change without notice and 
 * is not warranted to be error-free. If you find any errors, please report 
 * them to us in writing.
 *
 * If this is software or related software documentation that is delivered to 
 * the U.S. Government or anyone licensing it on behalf of the U.S. Government,
 * the following notice is applicable:
 *
 * U.S. GOVERNMENT RIGHTS Programs, software, databases, and related
 * documentation and technical data delivered to U.S. Government customers are
 * "commercial computer software" or "commercial technical data" pursuant to the
 * applicable Federal Acquisition Regulation and agency-specific supplemental 
 * regulations. As such, the use, duplication, disclosure, modification, and 
 * adaptation shall be subject to the restrictions and license terms set forth 
 * in the applicable Government contract, and, to the extent applicable by 
 * the terms of the Government contract, the additional rights set
 * forth in FAR 52.227-19, Commercial Computer Software License (December 2007).
 * Oracle America, Inc., 500 Oracle Parkway, Redwood City, CA 94065.
 *
 * This software or hardware is developed for general use in a variety of 
 * information management applications. It is not developed or intended for 
 * use in any inherently dangerous applications, including applications which 
 * may create a risk of personal injury. If you use this software or hardware 
 * in dangerous applications, then you shall be responsible to take all 
 * appropriate fail-safe, backup, redundancy, and other measures to ensure
 * its safe use. Oracle Corporation and its affiliates disclaim any liability
 * for any damages caused by use of this software or hardware in dangerous 
 * applications.
 *
 * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
 * Other names may be trademarks of their respective owners.
 *
 * AMD, Opteron, the AMD logo, and the AMD Opteron logo are trademarks or 
 * registered trademarks of Advanced Micro Devices. Intel and Intel Xeon are 
 * trademarks or trademarks of Intel Corporation. All SPARC trademarks are 
 * used under license and are trademarks or registered trademarks of SPARC 
 * International, Inc. UNIX is a registered trademark licensed through 
 * X/Open Company, Ltd.
 *
 * This software or hardware and documentation may provide access to or 
 * information on content, products, and services from third parties. Oracle 
 * Corporation and its affiliates are not responsible for and expressly 
 * disclaim all warranties of any kind with respect to third-party content,
 * products, and services. Oracle Corporation and its affiliates will not be
 * responsible for any loss, costs, or damages incurred due to your access to
 * or use of third-party content, products, or services.
 */ 

package com.sun.glassfish.dasrecovery;

import com.sun.enterprise.util.StringUtils;
import com.sun.enterprise.universal.process.LocalAdminCommand;
import com.sun.enterprise.universal.process.ProcessManagerException;
import com.sun.enterprise.universal.i18n.LocalStringsImpl;
import com.sun.enterprise.config.serverbeans.Config;
import com.sun.enterprise.config.serverbeans.Domain;
import com.sun.glassfish.quiesce.SuspendDomain;
import com.sun.glassfish.quiesce.ResumeDomain;
import org.glassfish.api.ActionReport;
import org.glassfish.api.admin.ServerEnvironment;
import org.glassfish.api.admin.AdminCommandContext;
import org.glassfish.api.admin.CommandRunner;
import org.jvnet.hk2.annotations.Service;
import org.jvnet.hk2.annotations.Inject;
import org.jvnet.hk2.config.ConfigSupport;
import org.jvnet.hk2.config.SingleConfigCode;
import org.jvnet.hk2.config.Transaction;
import org.jvnet.hk2.config.TransactionFailure;

import java.beans.PropertyVetoException;

import java.util.ArrayList;
import java.util.Date;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * This class implements the BackupManager .
 * The BackupManager invokes the local backup command backup-domain to backup
 * on a suspended DAS and resume the DAS based on the value of the 
 * active-backup-enabled attribute.
 * If active-backup-enabled is true an automatic backup will occur without 
 * first quiesceing the DAS.
 * Once the backup is done it will resume the DAS
 * @author Bhakti Mehta
 */
@Service
public class BackupManagerService implements BackupManager {


    @Inject(name= ServerEnvironment.DEFAULT_INSTANCE_NAME)
    Config config;

    @Inject
    Domain domain;

    @Inject
    CommandRunner runner;


    private static final LocalStringsImpl strings =
            new LocalStringsImpl(BackupManagerService.class);


    public void backup(BackupConfig backupConfig, AdminCommandContext context)  {

        final ActionReport report = context.getActionReport();
        final Logger logger = context.getLogger();
        final String configName = backupConfig.getName();
        ActionReport actionReport = context.getActionReport();
        if (!new Boolean(backupConfig.getActiveBackupEnabled())) {
            final SuspendDomain suspendDomain = (SuspendDomain) runner
                    .getCommand("suspend-domain", actionReport,
                                context.getLogger());
            if (suspendDomain == null ) {
                logger.log(Level.SEVERE, "severe.command.notfound",
                    "suspend-domain");
                report.setMessage(strings.get("command.notfound",
                                              "suspend-domain"));
                report.setActionExitCode(ActionReport.ExitCode.FAILURE);
                return;
            }
            logger.log(Level.INFO, "info.execute.suspend.domain", configName);
            suspendDomain.execute(context);
            if(actionReport.getActionExitCode().equals(
                ActionReport.ExitCode.FAILURE)) {
                logger.log(Level.SEVERE, "severe.suspend.non.zero.status",
                    new java.lang.Object[] {"suspend-domain",
                                            actionReport.getMessage()});
                report.setMessage(strings.get("non.zero.status.cause",
                    "suspend-domain", actionReport.getMessage()));
                report.setActionExitCode(ActionReport.ExitCode.FAILURE);
                return;
            }
        }

        logger.log(Level.INFO, "info.executing.backup", configName);
        ArrayList<String> command = new ArrayList<String>();
        command.add("--_force=true");
        //If user specifies his own dir for backup use that
        if (backupConfig.getBackupDir()!= null){
            command.add("--backupdir=" +backupConfig.getBackupDir());
        }
        //If user specifies configonly use that
        if (backupConfig.getConfigOnly()!= null){
            command.add("--_configonly=" + new Boolean(backupConfig.getConfigOnly()));
        }

        //Pass the --backupconfig option to the backup-domain command
        //This is because these are scheduled backups associated with a 
        //backupconfig
        //If the backupDir is not set backup is done to 
        //domainDir + backupconfig.
        //If backupDir is set then backup is done to backupDir + domainName +
        //backupconfig.
        command.add("--backupconfig=" + backupConfig.getName());

        //Pass on the recycle limit to the backup command.
        command.add("--_recyclelimit=" + backupConfig.getRecycleLimit());

        //Pass the domain name to the command.  More than one domain may
        //exist.
        command.add(domain.getName());

        LocalAdminCommand lac = new LocalAdminCommand("backup-domain",
                command.toArray(new String[command.size()]));
        StringBuilder readableCommand = new StringBuilder(
                "asadmin backup-domain ");
        for (String s : command) {
            readableCommand.append(s);
            readableCommand.append(" ");
        }

        int status = 0;
        try {
            status = lac.execute();
        } catch (ProcessManagerException e) {

            logger.log(Level.SEVERE, "severe.backup-domain.failed",
                e.getLocalizedMessage());
            String msg = strings.get("backup-domain.failed" );
            report.setActionExitCode(ActionReport.ExitCode.FAILURE);
            report.setMessage(msg +e.getLocalizedMessage());
            report.setFailureCause(e);
            //If there is a failure do not return resume the domain and return

        }

        updateBackupDate(logger, backupConfig, status);

        logger.log(Level.INFO, "info.execute.backup",
            new java.lang.Object[] {configName, status});
        if (status != 0) {
            String msg = strings.get("non.zero.status",
                    readableCommand);

            report.setMessage(msg);
            report.setActionExitCode(ActionReport.ExitCode.FAILURE);
            ////If there is a failure do not return resume the domain and return
            //return;

        }

        if (!new Boolean(backupConfig.getActiveBackupEnabled())) {
            final ResumeDomain resumeDomain = (ResumeDomain) runner
                    .getCommand("resume-domain", actionReport,
                    context.getLogger());
            if (resumeDomain == null ) {
                logger.log(Level.SEVERE, "severe.command.notfound",
                    "resume-domain");
                report.setMessage(strings.get("command.notfound", "resume-domain"));
                report.setActionExitCode(ActionReport.ExitCode.FAILURE);
                return;
            }
            logger.log(Level.INFO, "info.execute.resume.domain", configName);
            resumeDomain.execute(context);
            if(actionReport.getActionExitCode().equals(ActionReport.ExitCode.FAILURE)) {
                logger.log(Level.SEVERE, "severe.resume.non.zero.status",
                    new java.lang.Object[] {"resume-domain", actionReport.getMessage()});
                report.setMessage(strings.get("non.zero.status.cause",
                    "resume-domain", actionReport.getMessage()));
                report.setActionExitCode(ActionReport.ExitCode.FAILURE);
                return;
            }
        }
        report.setActionExitCode(ActionReport.ExitCode.SUCCESS);
        return;
    }

    /**
     * Update the time-of-last-backup-attempt and time-of-last-backup-success
     * properties.  This needs to be done in transaction.
     */
    private void updateBackupDate(Logger logger, BackupConfig backupConfig,
                                  final int status) {

        final long backupDate = new Date().getTime();

        try {
            ConfigSupport.apply(new SingleConfigCode<BackupConfig>() {

                public Object run(BackupConfig param) throws
                        PropertyVetoException,
                        TransactionFailure {

                    param.setTimeOfLastBackupAttempt(Long.toString(backupDate));

                    if (status == 0)
                        param.setTimeOfLastBackupSuccess(
                            Long.toString(backupDate));

                    return param;
                }
            }, backupConfig);
        } catch(TransactionFailure e) {
            logger.log(Level.WARNING, "warning.update.backup-config",
                backupConfig.getName());
            logger.warning("Unable to update backup-config last-backup-attempt/success time for: " + backupConfig.getName());
            logger.log(Level.WARNING, "TransactionFailure: " + e.toString());
            return;
        }
    }
}
