/*
 * Copyright (c) 2010, Oracle and/or its affiliates. All rights reserved.
 */
package com.sun.glassfish.dasrecovery;

import com.sun.enterprise.config.serverbeans.Config;
import com.sun.enterprise.util.StringUtils;
import com.sun.enterprise.v3.common.PropsFileActionReporter;

import java.util.Calendar;
import java.util.Date;
import java.util.SortedMap;
import java.util.Timer;
import java.util.TimerTask;
import java.util.TreeMap;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.Semaphore;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.glassfish.api.admin.AdminCommandContext;

/**
 * This class maintains the scheduling for backup configs. It uses a timer to
 * schedule the next backup to perform.
 * @author rickhall
 */
public class BackupScheduler
{
    private final Logger logger;
    private final Config config;
    private final BackupManager backupManager;
    private final Semaphore scheduleLock = new Semaphore(1);
    private BackupTimerTask currentTimerTask = null;
    private Timer timer = null;

    private static final Date startDate = Calendar.getInstance().getTime();

    /**
     * Constructs a backup scheduler with the specified logger, configuration
     * object, and backup manager.
     * @param logger the logger this scheduler should use.
     * @param config the configuration object used to obtain configuration data.
     * @param backupManager backup manager to use for performing backups.
     */
    public BackupScheduler(Logger logger, Config config, BackupManager backupManager) {
        this.logger = logger;
        this.config = config;
        this.backupManager = backupManager;
    }

    /**
     * This method stops the backup scheduler. If there is a backup in
     * progress, it will wait for it to finish.
     * @throws InterruptedException if the calling thread is interrupted.
     */
    public void stop() throws InterruptedException {
        // Grab local reference to avoid holding the lock.
        BackupTimerTask localTask = null;
        synchronized (this) {
            if (timer != null)
                timer.cancel();
            localTask = currentTimerTask;
        }
        // Wait for the local task to stop if it has started.
        if (localTask != null) {
            localTask.await();
        }
    }

    /**
     * Updates the backup schedule to the latest configuration data. If there
     * is no backup currently being processed, then it will cancel any pending
     * backup and recalculate the next backup based on the current configuration
     * information. If a backup is in progress, then it does not do anything
     * since the next backup will be recalculated after the executing backup
     * completes.
     */
    public synchronized void update() {
        // Try to acquire the schedule lock. If it is held by the backup task,
        // then ignore the update since the backup task will update the
        // schedule.
        if (scheduleLock.tryAcquire()) {
            scheduleNextBackup();
            scheduleLock.release();
        }
    }

    private void scheduleNextBackup() {
        // Look up current schedules and backups from the config.
        Schedules schedules = config.getExtensionByType(Schedules.class);
        BackupConfigs backups = config.getExtensionByType(BackupConfigs.class);

        // Clear existing task, since it will be recreated if necessary.
        currentTimerTask = null;

        // Create sorted set to determine nearest backup.
        SortedMap<Calendar, BackupConfig> sortedBackups =
            new TreeMap<Calendar, BackupConfig>();

        // Try to validate backups if any exist.
        if ((backups != null) && (schedules != null)) {

            // Walk the list of BackupConfig elements and access the schedule
            // for those that are enabled.
            for (BackupConfig backup : backups.getBackupConfigs()) {
                if (backup.getAutoBackupEnabled().equals("true")) {
                    if (validateBackupConfig(backup, schedules)) {
                        // Calculate next timeout and add it to sorted set. The
                        // next timeout is calculated based on the last attempted
                        // backup time. This will also account for situations
                        // where the server was stopped and we missed backups,
                        // which will result in missed backups being performed
                        // immediately and then revert to their normal schedule
                        // thereafter.

                        // Since new backup configs will not yet have a last
                        // attempted time, we'll use our start time as the default
                        // last attempted time. We cannot use the current time
                        // because we'll end up pushing new backup configs into
                        // the future when we recalculate timeouts when config
                        // events occur. Using the fixed start time ensures all
                        // new backup configs will eventually be scheduled.
                        Date last = startDate;
                        // Try to determine last backup time, if there was one,
                        // which is a Long encoded as a String.
                        String lastStr = backup.getTimeOfLastBackupAttempt();
                        if (lastStr != null) {
                            try {
                                // Parse last backup time. If the value is zero, then
                                // no backup has ever been performed, so just use the
                                // start time to calculate next timeout.
                                long lastLong = Long.parseLong(lastStr);
                                if (lastLong != 0) {
                                    last = new Date(lastLong);
                                }
                            } catch (NumberFormatException ex) {
                                logger.log(Level.WARNING,
                                    "warning.invalid.backup.time",
                                    new java.lang.Object[] {lastStr,
                                                            backup.getName()});
                            }
                        }
                        // Create timer schedule by converting schedule to an expression.
                        TimerSchedule timerSchedule =
                            new TimerSchedule(
                                toScheduleExpression(
                                    schedules.getScheduleByName(backup.getSchedule())));
                        // Sort the backup based on next timeout value.
                        sortedBackups.put(
                            timerSchedule.getNextTimeout(last),
                            backup);
                        if (logger.isLoggable(Level.INFO)) {
                            logger.log(Level.INFO, "info.enable.backup",
                                new java.lang.Object[] {backup.getName(),
                                                        backup.getSchedule()});
                        }
                    } else {
                        // The named BackupConfig/Schedule could not be validated.
                        logger.log(Level.WARNING, "warning.backup.validation",
                            backup.getName()); 
                    }
                }
            }
        }

        // If we have any backup configs, then create a timer and
        // schedule the nearest backup.
        if (!sortedBackups.isEmpty()) {
            if (timer == null) {
                timer = new Timer("DAS Backup");
            }
            // Get backup with nearest timeout and schedule it.
            Calendar firstKey = sortedBackups.firstKey();
            BackupConfig backup = sortedBackups.get(firstKey);
            currentTimerTask = new BackupTimerTask(backup);
            timer.schedule(currentTimerTask, firstKey.getTime());
            if (logger.isLoggable(Level.FINE)) {
                logger.log(Level.FINE,
                    "BackupScheduler: Next backup is "
                    + backup.getName() + " at " + firstKey.getTime());
            }
        } else if (sortedBackups.isEmpty()) {
            // If we have no backups, then cancel the timer.
            if (timer != null) {
                timer.cancel();
                timer = null;
            }

            if (logger.isLoggable(Level.FINE)) {
                if ((backups == null) || backups.getBackupConfigs().isEmpty()) {
                    logger.log(Level.FINE,
                        "BackupScheduler: Not enabled. No backups.");
                } else if ((schedules == null) || schedules.getSchedules().isEmpty()) {
                    logger.log(Level.FINE,
                        "BackupScheduler: Not enabled. No schedules.");
                } else {
                    logger.log(Level.FINE,
                        "BackupScheduler: Not enabled. No valid or enabled backups.");
                }
            }
        }
    }

    /**
     * Validates a backup configuration by making sure its schedule exists.
     * @param bc the backup configuration to validate.
     * @param scheds the available schedules.
     * @return <tt>true</tt> if the backup configuration is valid,
     *         <tt>false</tt> otherwise.
     */
    private static boolean validateBackupConfig(BackupConfig bc, Schedules scheds) {
        String schedName = bc.getSchedule();
        if (StringUtils.ok(schedName)) {
            Schedule sched = scheds.getScheduleByName(schedName);
            if (sched == null) {
                return false;
            }
        } else {
            return false;
        }
        return true;
    }

    /**
     * Converts a schedule to a schedule expression.
     * @param schedule the schedule to convert.
     * @return the converted schedule expression.
     */
    private static ScheduleExpression toScheduleExpression(Schedule schedule) {
        ScheduleExpression expression = new ScheduleExpression();
        expression.second(schedule.getSecond());
        expression.minute(schedule.getMinute());
        expression.hour(schedule.getHour());
        expression.dayOfMonth(schedule.getDayOfMonth());
        expression.month(schedule.getMonth());
        expression.dayOfWeek(schedule.getDayOfWeek());
        expression.year(schedule.getYear());
        return expression;
    }

    /**
     * This class implements a timer task for performing backups. There will
     * only ever be one of these tasks in the timer at a time. When this task
     * completes its backup, it schedules a new timer task at that point. It
     * also implements a latch method that can be used to wait for the task
     * to complete if it has started.
     */
    private class BackupTimerTask extends TimerTask {
        private final BackupConfig backup;
        private CountDownLatch latch;

        BackupTimerTask(BackupConfig backup) {
            this.backup = backup;
        }

        /**
         * Waits for this task to complete if it has started. If the
         * task hasn't started, then this method returns immediately.
         * @throws InterruptedException if the calling thread is interrupted.
         */
        public void await() throws InterruptedException {
            // Grab a local reference to the latch to avoid
            // holding the lock if we wait on it.
            CountDownLatch localLatch = null;
            synchronized (this) {
                localLatch = latch;
            }
            if (localLatch != null) {
                localLatch.await();
            }
        }

        @Override
        public void run() {
            CountDownLatch localLatch = null;
            try {
                // Grab the object lock to access shared members.
                synchronized (BackupScheduler.this) {
                    // If this isn't the current task, then it has been
                    // canceled, so do nothing.
                    if (currentTimerTask != this) {
                        return;
                    }

                    // Create a latch so we can indicate when this task
                    // is complete.
                    localLatch = latch = new CountDownLatch(1);

                    // Acquire the schedule lock, since we'll need to update
                    // the schedule after performing the current backup and
                    // we want to defer any scheduling changes by other threads
                    // until after this backup has completed to avoid race
                    // conditions. Note, we will never block here since we
                    // had to acquire the object lock first; this is one side
                    // of a hand-shake locking protocol with threads calling
                    // the update() method above.
                    try {
                        scheduleLock.acquire();
                    } catch (InterruptedException ex) {
                        if (logger.isLoggable(Level.FINE)) {
                            logger.log(Level.FINE,
                                "BackupScheduler: Not performing "
                                    + backup.getName()
                                + " backup on "
                                    + backup.getSchedule()
                                + " schedule due to interrupt.");
                        }
                    }
                }
                
                // This is where we do the backup.

                if (logger.isLoggable(Level.INFO)) {
                    logger.log(Level.INFO, "info.started.backup",
                        new java.lang.Object[] {backup.getName(),
                                                backup.getSchedule()});
                }
                if (backupManager != null) {
                    backupManager.backup(
                        backup,
                        new AdminCommandContext(
                            logger, new PropsFileActionReporter()));
                }
                if (logger.isLoggable(Level.INFO)) {
                    logger.log(Level.INFO, "info.completed.backup",
                        backup.getName());
                }

                // Grab the object lock again, so we can update the schedule
                // and release the schedule lock.
                synchronized (BackupScheduler.this) {
                    scheduleNextBackup();
                    scheduleLock.release();
                }
            } finally {
                // No matter how we exit this method, we want to
                // open our latch if we have one.
                if (localLatch != null) {
                    localLatch.countDown();
                    localLatch = null;
                }
            }
        }
    }
}
