/*
 * Copyright (c) 2010, Oracle and/or its affiliates. All rights reserved.
 */

package com.sun.glassfish.dasrecovery;


import org.glassfish.api.I18n;
import org.glassfish.api.ActionReport;
import org.glassfish.api.ActionReport.ExitCode;
import org.glassfish.api.admin.*;
import org.glassfish.api.Param;
import org.glassfish.api.admin.config.ModelBinding;

import com.sun.enterprise.config.serverbeans.Config;
import com.sun.enterprise.universal.i18n.LocalStringsImpl;
import com.sun.enterprise.util.StringUtils;

import org.jvnet.hk2.annotations.Service;
import org.jvnet.hk2.annotations.Inject;
import org.jvnet.hk2.annotations.Scoped;
import org.jvnet.hk2.component.PerLookup;
import org.jvnet.hk2.component.Habitat;
import org.jvnet.hk2.config.ConfigSupport;
import org.jvnet.hk2.config.SingleConfigCode;
import org.jvnet.hk2.config.TransactionFailure;

import java.beans.PropertyVetoException;

/**
 * Remote asadmin command: create-backup-config
 *
 * Purpose: Allows the invoker to configure automatic domain
 * backup. 
 *
 * Implementation: This remote command ...
 *
 * @author Chris Kasso
 *
 */

@Service(name="create-backup-config")
@ExecuteOn(RuntimeType.DAS)  // Only appropriate for the DAS
@Scoped(PerLookup.class)
@I18n("backup.config.command")
public class CreateBackupConfig implements AdminCommand {

    @Inject(name=ServerEnvironment.DEFAULT_INSTANCE_NAME)
    Config config;

    @Inject
    private Habitat habitat;

    @I18n("backup.config.schedule")
    @ModelBinding(type=BackupConfig.class, getterMethodName="getSchedule")
    @Param(optional = true)
    private String schedule;

    @I18n("backup.config.directory")
    @Param(optional = true)
    //@ModelBinding(type=BackupConfig.class, getterMethodName="getBackupDir")
    private String backupDir;

    @I18n("backup.config.recyclelimit")
    @ModelBinding(type=BackupConfig.class, getterMethodName="getRecycleLimit")
    @Param(optional = true)
    private String recycleLimit;

    @I18n("backup.config.configonly")
    @ModelBinding(type=BackupConfig.class, getterMethodName="getConfigOnly")
    @Param(optional = true, defaultValue = "false")
    private boolean configOnly;

    @I18n("backup.config.name")
    @Param(optional = false, primary = true)
    private String name;

    @I18n("backup.config.activebackupenabled")
    @ModelBinding(type=BackupConfig.class, getterMethodName="getActiveBackupEnabled")
    @Param(optional = true, defaultValue = "false")
    private boolean activeBackupEnabled;

    @ModelBinding(type=BackupConfig.class, getterMethodName="getAutoBackupEnabled")
    @Param(optional = true, defaultValue = "true")
    private boolean autoBackupEnabled;

    private static final LocalStringsImpl strings =
            new LocalStringsImpl(CreateBackupConfig.class);


    public void execute(AdminCommandContext context) {

        ActionReport report = context.getActionReport();

        BackupConfigs backupConfigs = config.getExtensionByType(
                BackupConfigs.class);

        // Validate the schedule exists. 
        if (StringUtils.ok(schedule)) {
            String msg = generateMissingSchedWarning(schedule); 
            if (msg != null) {
                report.setMessage(msg);
                report.setActionExitCode(ActionReport.ExitCode.FAILURE);
                return;
            }
        }
        
        /**
         * The backup-configs does not exist in this Config.  We will need to
         * add it.
         */
        if (backupConfigs == null) {
            try {
                ConfigSupport.apply(new SingleConfigCode<Config>() {

                    public Object run(Config parent) throws TransactionFailure {
                        BackupConfigs child =
                                parent.createChild(BackupConfigs.class);

                        parent.getContainers().add(child);
                        return child;
                    }
                }, config);
            } catch(TransactionFailure e) {
                report.setMessage(
                        strings.get("backup.create.createconfig.failure"));
                report.setActionExitCode(ActionReport.ExitCode.FAILURE);
                report.setFailureCause(e);
                return;
            }

            backupConfigs = config.getExtensionByType(BackupConfigs.class);
            if (backupConfigs == null) {
                report.setMessage(
                        strings.get("XXX: backupConfigs is null"));
                report.setActionExitCode(ActionReport.ExitCode.FAILURE);
                return;
            }

        } else {
            /**
             * If the backup-configs exist see if the named config already
             * exists.  If it does then we will generate an error.
             * In the future we will allow the backup-config to be updated.
             */
            if (backupConfigs.getBackupConfigByName(name) != null) {
                report.setMessage(
                        strings.get("backup.create.nameexists.failure",
                                name));
                report.setActionExitCode(ActionReport.ExitCode.FAILURE);
                return;
            }
        }

        //XXX: How do we handle the PropertyVetoException?

        try {
            ConfigSupport.apply(new SingleConfigCode<BackupConfigs>() {

                public Object run(BackupConfigs param) throws
                        TransactionFailure,
                        PropertyVetoException {

                    BackupConfig child =
                            param.createChild(BackupConfig.class);

                    if (StringUtils.ok(name))
                        child.setName(name);
                    if (StringUtils.ok(schedule))
                        child.setSchedule(schedule);
                    if (StringUtils.ok(backupDir))
                        child.setBackupDir(backupDir);
                    if (StringUtils.ok(recycleLimit))
                        child.setRecycleLimit(recycleLimit);
                    child.setConfigOnly(
                            (configOnly) ? "true" : "false");
                    if (StringUtils.ok(recycleLimit))
                        child.setAutoBackupEnabled("false");
                    child.setAutoBackupEnabled(
                            (autoBackupEnabled)?"true":"false");
                    child.setActiveBackupEnabled(
                            (activeBackupEnabled)?"true":"false");

                    param.getBackupConfigs().add(child);
                    return child;
                }
            }, backupConfigs);
        } catch(TransactionFailure e) {
            report.setMessage(
                    strings.get("backup.create.createconfig.failure"));
            report.setActionExitCode(ActionReport.ExitCode.FAILURE);
            report.setFailureCause(e);
            return;
        }

        report.setActionExitCode(ActionReport.ExitCode.SUCCESS);
        return;
    }

    /**
     * Determine if the provided schedule name exists as an existing
     * Schedule.  It is not an error if it is missing but we warn
     * the user that the condition exists.
     */
    private String generateMissingSchedWarning(String scheduleName) { 

        Schedules schedules = config.getExtensionByType(Schedules.class);

        if (schedules == null) 
            return strings.get("backup.create.nosched", scheduleName);

        /**
         * Find a specific schedule.
         */
        Schedule sched = schedules.getScheduleByName(scheduleName);

        // The named schedule can not be found.
        if (sched == null) 
            return strings.get("backup.create.nosched", scheduleName);
        else
            return null;
    }
}
