/*
 * Copyright (c) 2010, Oracle and/or its affiliates. All rights reserved.
 */

package com.sun.glassfish.dasrecovery;


import org.glassfish.api.I18n;
import org.glassfish.api.ActionReport;
import org.glassfish.api.ActionReport.ExitCode;
import org.glassfish.api.admin.*;
import org.glassfish.api.Param;
import org.glassfish.api.admin.config.ModelBinding;

import com.sun.enterprise.config.serverbeans.Config;
import com.sun.enterprise.universal.i18n.LocalStringsImpl;
import com.sun.enterprise.util.StringUtils;

import org.jvnet.hk2.annotations.Service;
import org.jvnet.hk2.annotations.Inject;
import org.jvnet.hk2.annotations.Scoped;
import org.jvnet.hk2.component.PerLookup;
import org.jvnet.hk2.component.Habitat;
import org.jvnet.hk2.config.ConfigSupport;
import org.jvnet.hk2.config.SingleConfigCode;
import org.jvnet.hk2.config.TransactionFailure;

import java.beans.PropertyVetoException;
import java.util.BitSet;

/**
 * Remote asadmin command: create-schedule
 * 
 * Purpose: Allows the invoker to create a schedule.
 * 
 * 
 * Implementation: This remote command ...
 *
 * @author Chris Kasso
 *
 */

@Service(name="create-schedule")
@ExecuteOn(RuntimeType.DAS)  // Only appropriate for the DAS
@Scoped(PerLookup.class)
@I18n("schedule.create.command")
public class CreateSchedule implements AdminCommand {

    @Inject(name=ServerEnvironment.DEFAULT_INSTANCE_NAME)
    Config config; 

    @Inject
    private Habitat habitat;

    @I18n("schedule.create.name")
    @Param(primary = true, optional = false)
    private String name;

    @I18n("schedule.create.second")
    @ModelBinding(type=Schedule.class, getterMethodName ="getSecond")
    @Param(optional = true)
    private String second;

    @I18n("schedule.create.minute")
    @ModelBinding(type=Schedule.class, getterMethodName ="getMinute")
    @Param(optional = true)
    private String minute;

    @I18n("schedule.create.hour")
    @ModelBinding(type=Schedule.class, getterMethodName ="getHour")
    @Param(optional = true)
    private String hour;

    @I18n("schedule.create.dayofmonth")
    @ModelBinding(type=Schedule.class, getterMethodName ="getDayOfMonth")
    @Param(alias = "day-of-month", optional = true)
    private String dayOfMonth;

    @I18n("schedule.create.month")
    @ModelBinding(type=Schedule.class, getterMethodName ="getMonth")
    @Param(optional = true)
    private String month;

    @I18n("schedule.create.dayofweek")
    @ModelBinding(type=Schedule.class, getterMethodName ="getDayOfWeek")
    @Param(alias = "day-of-week", optional = true)
    private String dayOfWeek;

    @I18n("schedule.create.year")
    @ModelBinding(type=Schedule.class, getterMethodName ="getYear")
    @Param(optional = true)
    private String year;

    private static final LocalStringsImpl strings =
            new LocalStringsImpl(CreateSchedule.class);


    public void execute(AdminCommandContext context) {

        ActionReport report = context.getActionReport();

        Schedules schedules = config.getExtensionByType(Schedules.class);

        /**
         * The schedules does not exist in this Config.  We will need to
         * add it plus the default schedules.
         */
        if (schedules == null) { 
            try {
                ConfigSupport.apply(new SingleConfigCode<Config>() {
        
                    public Object run(Config parent) throws TransactionFailure {

                        Schedules child = parent.createChild(Schedules.class);

                        //XXX: This is where we should create the default
                        //     schedules: daily, weekly, monthly.

                        parent.getContainers().add(child);
                        return child;
                    }
                }, config); 
            } catch(TransactionFailure e) {
                report.setMessage(
                           strings.get("schedule.create.createconfig.failure"));
                report.setActionExitCode(ActionReport.ExitCode.FAILURE);
                report.setFailureCause(e);
                return;
            }

            schedules = config.getExtensionByType(Schedules.class);
            if (schedules == null) { 
                report.setMessage(
                           strings.get("schedules is null"));
                report.setActionExitCode(ActionReport.ExitCode.FAILURE);
                return;
            }
        } else {
            /**
             * If the schedules exist see if the named schedule already
             * exists.  If it does then we will generate an error.
             * In the future we will allow the schedule to be updated.
             */
            if (schedules.getScheduleByName(name) != null) {
                report.setMessage(
                           strings.get("schedule.create.nameexists.failure",
                                       name));
                report.setActionExitCode(ActionReport.ExitCode.FAILURE);
                return;
            }
        }

        /**
         * Validate the values are acceptable.  This uses the methods of
         * TimerSchedule which validates the schedule relative to the
         * EJB Timer Service spec so it allows a bit more than what
         * our current implementation supports.   
         */
        try {
            BitSet bits = new BitSet(60); // unused
            TimerSchedule ts = new TimerSchedule();

            if (StringUtils.ok(second))
                ts.parseNumbersOrNames(second, bits, 0, 60, true, "second");
            if (StringUtils.ok(minute))
                ts.parseNumbersOrNames(minute, bits, 0, 60, true, "minute");
            if (StringUtils.ok(hour))
                ts.parseNumbersOrNames(hour, bits, 0, 24, true, "hour");
            if (StringUtils.ok(dayOfMonth))
                ts.parseDaysOfMonth(dayOfMonth);
            if (StringUtils.ok(month))
                ts.parseNumbersOrNames(month, bits, 1, 12, false, "month");
            if (StringUtils.ok(dayOfWeek))
                ts.parseNumbersOrNames(dayOfWeek, bits, 0, 7, false, "dayOfWeek");
            if (StringUtils.ok(year))
                ts.parseYears(year);
        } catch (IllegalArgumentException e) {
            report.setMessage(e.getMessage());
            report.setActionExitCode(ActionReport.ExitCode.FAILURE);
            return;
        }

        try {
            ConfigSupport.apply(new SingleConfigCode<Schedules>() {

                public Object run(Schedules param) throws 
                                                       TransactionFailure,
                                                       PropertyVetoException {

                        Schedule child = param.createChild(Schedule.class);

                        if (StringUtils.ok(name))
                            child.setName(name);
                        if (StringUtils.ok(second))
                            child.setSecond(second);
                        if (StringUtils.ok(minute))
                            child.setMinute(minute);
                        if (StringUtils.ok(hour))
                            child.setHour(hour);
                        if (StringUtils.ok(dayOfMonth))
                            child.setDayOfMonth(dayOfMonth);
                        if (StringUtils.ok(month))
                            child.setMonth(month);
                        if (StringUtils.ok(dayOfWeek))
                            child.setDayOfWeek(dayOfWeek);
                        if (StringUtils.ok(year))
                            child.setYear(year);

                        param.getSchedules().add(child);
                        return child;
                }
            }, schedules); 
        } catch(TransactionFailure e) {
            report.setMessage(
                           strings.get("schedule.create.createconfig.failure"));
                report.setActionExitCode(ActionReport.ExitCode.FAILURE);
                report.setFailureCause(e);
                return;
        }

        report.setActionExitCode(ActionReport.ExitCode.SUCCESS);
        return;
    }
}
