/*
 * Copyright (c) 2010, Oracle and/or its affiliates. All rights reserved.
 */

package com.sun.glassfish.dasrecovery;


import org.glassfish.api.I18n;
import org.glassfish.api.ActionReport;
import org.glassfish.api.ActionReport.ExitCode;
import org.glassfish.api.admin.*;
import org.glassfish.api.Param;

import com.sun.enterprise.config.serverbeans.Config;
import com.sun.enterprise.universal.i18n.LocalStringsImpl;
import com.sun.enterprise.util.StringUtils;

import org.jvnet.hk2.annotations.Service;
import org.jvnet.hk2.annotations.Inject;
import org.jvnet.hk2.annotations.Scoped;
import org.jvnet.hk2.component.PerLookup;
import org.jvnet.hk2.component.Habitat;
import org.jvnet.hk2.config.ConfigSupport;
import org.jvnet.hk2.config.SingleConfigCode;
import org.jvnet.hk2.config.TransactionFailure;


/**
 * Remote asadmin command: delete-schedule
 * 
 * Purpose: Allows the invoker to delete a schedule.
 * 
 * 
 * @author Chris Kasso
 *
 */

@Service(name="delete-schedule")
@ExecuteOn(RuntimeType.DAS)  // Only appropriate for the DAS
@Scoped(PerLookup.class)
@I18n("schedule.delete.command")
public class DeleteSchedule implements AdminCommand {

    @Inject(name=ServerEnvironment.DEFAULT_INSTANCE_NAME)
    Config config; 

    @Inject
    private Habitat habitat;

    @I18n("schedule.delete.name")
    @Param(primary = true, optional = false)
    private String name;

    private static final LocalStringsImpl strings =
            new LocalStringsImpl(DeleteSchedule.class);


    public void execute(AdminCommandContext context) {

        ActionReport report = context.getActionReport();

	Schedules schedules = config.getExtensionByType(Schedules.class);

        /**
         * Check to see if schedules exists in this Config. 
         */
        if (schedules == null) { 
	    report.setMessage(
                           strings.get("schedule.delete.notfound"));
            report.setActionExitCode(ActionReport.ExitCode.FAILURE);
            return;
        }

        /**
         * Find a specific schedule.
         */
        if (!StringUtils.ok(name)) {
	    report.setMessage(
                           strings.get("schedule.delete.badname"));
            report.setActionExitCode(ActionReport.ExitCode.FAILURE);
            return;
        }

        Schedule sched = schedules.getScheduleByName(name);
 
        // The named schedule can not be found.
        if (sched == null) {
            report.setMessage(
                           strings.get("schedule.delete.notfound", name));
            report.setActionExitCode(ActionReport.ExitCode.FAILURE);
            return;
        }

        /**
         * We don't allow a schedule to be deleted if someone else depends 
         * on it.
         */
        StringBuffer bcNames = new StringBuffer(); 
        if (scheduleInUse(name, bcNames)) {
	    report.setMessage(strings.get("schedule.inuse", bcNames));
            report.setActionExitCode(ActionReport.ExitCode.FAILURE);
            return;
        }

	try {
            ConfigSupport.apply(new SingleConfigCode<Schedules>() {

                public Object run(Schedules param) throws TransactionFailure {
                        Schedule schedule = param.getScheduleByName(name);
                        param.getSchedules().remove(schedule);
                        return param;
                }
            }, schedules); 
	} catch(TransactionFailure e) {
	    report.setMessage(
                           strings.get("schedule.delete.failure", name));
            report.setActionExitCode(ActionReport.ExitCode.FAILURE);
            report.setFailureCause(e);
            return;
        }

        report.setActionExitCode(ActionReport.ExitCode.SUCCESS);
        return;
    }

    /**
     * Determine if any backup-configs reference this schedule.
     */
    private boolean scheduleInUse(String name, StringBuffer bcNames) {
        boolean bcExists = false;
        boolean needComma = false;

        BackupConfigs backupConfigs = 
                config.getExtensionByType(BackupConfigs.class);

        /**
         * Check if any backupConfigs exist in this Config.
         */
        if (backupConfigs == null) {
            return false;
        }

        /**
         * Walk the backup-configs looking for the named schedule.
         */
        for (BackupConfig backupConfig : backupConfigs.getBackupConfigs())  {
            if (backupConfig.getSchedule().equals(name)) {
                if (needComma)
                    bcNames.append(", ");
                bcNames.append(backupConfig.getName());
                needComma = true;
                bcExists = true;
            }
        }
        return bcExists;
    }
}
