/*
 * Copyright (c) 2010, Oracle and/or its affiliates. All rights reserved.
 */
package com.sun.glassfish.dasrecovery;

import java.beans.PropertyChangeEvent;
import java.util.logging.Level;
import java.util.logging.Logger;

import com.sun.logging.LogDomains;

import com.sun.enterprise.config.serverbeans.Config;
import com.sun.enterprise.universal.i18n.LocalStringsImpl;
import com.sun.enterprise.v3.common.PropsFileActionReporter;
import java.util.ArrayList;
import org.glassfish.api.ActionReport;
import org.glassfish.api.admin.AdminCommandContext;

import org.glassfish.api.admin.ServerEnvironment;
import org.glassfish.api.admin.ExecuteOn;
import org.glassfish.api.admin.RuntimeType;
import org.glassfish.internal.api.PostStartup;

import org.jvnet.hk2.annotations.Inject;
import org.jvnet.hk2.annotations.Scoped;
import org.jvnet.hk2.annotations.Service;
import org.jvnet.hk2.component.Singleton;
import org.jvnet.hk2.component.PostConstruct;
import org.jvnet.hk2.component.PreDestroy;
import org.jvnet.hk2.config.ConfigListener;
import org.jvnet.hk2.config.ConfigSupport;
import org.jvnet.hk2.config.ObservableBean;
import org.jvnet.hk2.config.SingleConfigCode;
import org.jvnet.hk2.config.TransactionFailure;
import org.jvnet.hk2.config.UnprocessedChangeEvents;

import java.beans.PropertyVetoException;

/**
 * This service is started when the DAS is started.  It determines if
 * automatic domain backup should be performed and if so establishes
 * the timer which will perform the real work.
 *
 *
 * @author Chris Kasso
 *
 */
@Scoped(Singleton.class)
@Service()
public class DomainBackupService
    implements PostStartup, PostConstruct, PreDestroy {

    @Inject
    private ServerEnvironment env;

    @Inject(name = ServerEnvironment.DEFAULT_INSTANCE_NAME)
    Config config;

    final static Logger logger = 
        LogDomains.getLogger(DomainBackupService.class, 
                             "javax.enterprise.system.tools.backup");

    @Inject
    BackupManager backupManager;

    volatile BackupScheduler scheduler;

    private static final LocalStringsImpl strings =
        new LocalStringsImpl(DomainBackupService.class);

    @Override
    public void postConstruct() {
        ActionReport report = new AdminCommandContext(
            logger, new PropsFileActionReporter()).getActionReport();

        // This service is only execute in the context of the DAS. (14649)
        if (!env.isDas())
            return;

        // Get the current configs and schedules, create if necessary.
        Schedules schedules = createSchedules(report);
        if (schedules == null) {
            return;
        }

        BackupConfigs backups = createBackupConfigs(report);
        if (backups == null) {
            return;
        }

        logger.fine("Initializing Domain Backup Service."); 

        // Listen on schedules and backups for config changes.
        ConfigListener cl = new ConfigListenerImpl();
        ObservableBean bean = (ObservableBean) ConfigSupport.getImpl(schedules);
        bean.addListener(cl);
        bean = (ObservableBean) ConfigSupport.getImpl(backups);
        bean.addListener(cl);

        // Create a scheduler and initialize it with the current configuration.
        scheduler = new BackupScheduler(logger, config, backupManager);
        scheduler.update();
    }

    @Override
    public void preDestroy() {

        // This service is only execute in the context of the DAS. (14649)
        if (!env.isDas())
            return;

        if (scheduler != null) {
            try {
                scheduler.stop();
            } catch (Exception ex) {
                logger.log(Level.WARNING, "warning.stopping.scheduler", ex);
            }
        }
    }

    private Schedules createSchedules(ActionReport report) {
        Schedules schedules = config.getExtensionByType(Schedules.class);

        // Create Schedules if it doesn't exist.
        if (schedules == null) {
            try {
                ConfigSupport.apply(new SingleConfigCode<Config>() {
                    public Object run(Config parent) throws TransactionFailure {
                        Schedules child = parent.createChild(Schedules.class);
                        parent.getContainers().add(child);
                        return child;
                    }
                }, config);
            } catch(TransactionFailure e) {
                report.setMessage(
                strings.get("schedule.create.createconfig.failure"));
                report.setActionExitCode(ActionReport.ExitCode.FAILURE);
                report.setFailureCause(e);
                return null;
            }

            schedules = config.getExtensionByType(Schedules.class);
            if (schedules == null) {
                report.setMessage(
                strings.get("schedules is null"));
                report.setActionExitCode(ActionReport.ExitCode.FAILURE);
                return null;
            } else {
                manageDefaultSchedules(schedules);
            }
        }
        return schedules;
    }

    private BackupConfigs createBackupConfigs(ActionReport report) {
        BackupConfigs backups = config.getExtensionByType(BackupConfigs.class);

        // Create BackupConfigs if it doesn't exist.
        if (backups == null) {
            try {
                ConfigSupport.apply(new SingleConfigCode<Config>() {
                    public Object run(Config parent) throws TransactionFailure {
                        BackupConfigs child = parent.createChild(BackupConfigs.class);
                        parent.getContainers().add(child);
                        return child;
                    }
                }, config);
            } catch(TransactionFailure e) {
                report.setMessage(
                strings.get("backup.create.createconfig.failure"));
                report.setActionExitCode(ActionReport.ExitCode.FAILURE);
                report.setFailureCause(e);
                return null;
            }

            backups = config.getExtensionByType(BackupConfigs.class);
            if (backups == null) {
                report.setMessage(
                strings.get("backupConfigs is null"));
                report.setActionExitCode(ActionReport.ExitCode.FAILURE);
                return null;
            }
        }
        return backups;
    }

    private void manageDefaultSchedules(Schedules schedules) {

        if (schedules.getScheduleByName("daily") == null) {
            createSchedule(schedules, 
                "daily",   // Name
                "0",       // Second
                "0",       // Minute
                "0",       // Hour
                "*",       // Day Of Month
                "*",       // Month
                "*",       // Day Of Week
                "*"        // Year
                );
        }
        if (schedules.getScheduleByName("weekly") == null) {
            createSchedule(schedules, 
                "weekly",  // Name
                "0",       // Second
                "0",       // Minute
                "0",       // Hour
                "*",       // Day Of Month
                "*",       // Month
                "Sun",     // Day Of Week
                "*"        // Year
                );
        }
        if (schedules.getScheduleByName("monthly") == null) {
            createSchedule(schedules, 
                "monthly", // Name
                "0",       // Second
                "0",       // Minute
                "0",       // Hour
                "1",       // Day Of Month
                "*",       // Month
                "*",       // Day Of Week
                "*"        // Year
                );
        }
    }

    private void createSchedule(Schedules schedules,
                                final String name, 
                                final String second,
                                final String minute,
                                final String hour,
                                final String dayOfMonth,
                                final String month,
                                final String dayOfWeek,
                                final String year) {
        try {
            ConfigSupport.apply(new SingleConfigCode<Schedules>() {

                public Object run(Schedules param) throws
                                                       TransactionFailure,
                                                       PropertyVetoException {

                        Schedule child = param.createChild(Schedule.class);

                        child.setName(name);
                        child.setSecond(second);
                        child.setMinute(minute);
                        child.setHour(hour);
                        child.setDayOfMonth(dayOfMonth);
                        child.setMonth(month);
                        child.setDayOfWeek(dayOfWeek);
                        child.setYear(year);

                        param.getSchedules().add(child);
                        return child;
                }
            }, schedules);
        } catch(TransactionFailure e) {
            logger.log(Level.WARNING, "warning.create.default.schedule", e);
            return;
        }
    }

    private class ConfigListenerImpl implements ConfigListener {

        /**
         * This method is notified for any changes in schedules or
         * backup configs. Any change results in the backup scheduler
         * being updated.
         * @param pces the configuration change events.
         * @return the unprocessed change events.
         */
        @Override
        public UnprocessedChangeEvents changed(PropertyChangeEvent[] pces) {
            if (logger.isLoggable(Level.FINE)) {
                int i = 0;
                for (PropertyChangeEvent pce : pces) {
                    logger.log(Level.FINE,
                        "DomainBackupService: Child change: "
                        + (i++) + ". "
                        + " PROP ID " + pce.getPropagationId() + " "
                        + pce.getPropertyName()
                        + " NEW "
                        + pce.getNewValue()
                        + " (" + ((pce.getNewValue() == null) ? "null" : pce.getNewValue().getClass().getName())
                        + ") OLD "
                        + pce.getOldValue()
                        + " (" + ((pce.getOldValue() == null) ? "null" : pce.getOldValue().getClass().getName())
                        );
                }
            }

            // Notify the scheduler to update its configuration.
            scheduler.update();

            // TODO: RSH - We should just return null, but this was causing
            //       a QL failure in AMXProxyTests.testAutoConvert().
            return new UnprocessedChangeEvents(new ArrayList());
        }
    }
}
