/*
 * Copyright (c) 2010, Oracle and/or its affiliates. All rights reserved.
 */

package com.sun.glassfish.dasrecovery;

import java.util.logging.Level;
import java.util.logging.Logger;

import org.glassfish.api.I18n;
import org.glassfish.api.ActionReport;
import org.glassfish.api.ActionReport.ExitCode;
import org.glassfish.api.admin.*;
import org.glassfish.api.Param;

import com.sun.enterprise.config.serverbeans.Config;
import com.sun.enterprise.universal.i18n.LocalStringsImpl;
import com.sun.enterprise.util.StringUtils;

import org.jvnet.hk2.annotations.Service;
import org.jvnet.hk2.annotations.Inject;
import org.jvnet.hk2.annotations.Scoped;
import org.jvnet.hk2.component.PerLookup;
import org.jvnet.hk2.component.Habitat;
import org.jvnet.hk2.config.ConfigSupport;
import org.jvnet.hk2.config.SingleConfigCode;
import org.jvnet.hk2.config.Transaction;
import org.jvnet.hk2.config.TransactionFailure;

import java.beans.PropertyVetoException;

/**
 * Remote asadmin command: enable-backup-config
 *
 * Purpose: Allows the invoker to enable automatic domain
 * backup. 
 *
 * Implementation: This remote command ...
 *
 * @author Chris Kasso
 *
 */

@Service(name="enable-backup-config")
@ExecuteOn(RuntimeType.DAS)  // Only appropriate for the DAS
@Scoped(PerLookup.class)
@I18n("backup.command.enable")
public class EnableBackupConfigCommand implements AdminCommand {


    @Inject(name=ServerEnvironment.DEFAULT_INSTANCE_NAME)
    Config config;

    @Inject
    DomainBackupService dbs;

    @Inject
    private Habitat habitat;

    @Inject
    Logger logger;

    @I18n("backup.config.name")
    @Param(optional = false, primary = true)
    private String name;


    private static final LocalStringsImpl strings =
            new LocalStringsImpl(EnableBackupConfigCommand.class);


    public void execute(AdminCommandContext context) {

        ActionReport report = context.getActionReport();

        BackupConfigs backupConfigs =
                config.getExtensionByType(BackupConfigs.class);

        /**
         * Check if any backupConfigs exist in this Config.
         */
        if (backupConfigs == null) {
            // No backup configs so this is an error.
            report.setMessage(strings.get("backup.notfound", name));
            report.setActionExitCode(ActionReport.ExitCode.FAILURE);
            return;
        }

        /**
         * Find the specific backup config.
         */
        if (!StringUtils.ok(name)) {
            report.setMessage(strings.get("backup.badname"));
            report.setActionExitCode(ActionReport.ExitCode.FAILURE);
            return;
        }


        /**
         * The named backupConfig can not be found.
         */
        if (backupConfigs.getBackupConfigByName(name) == null) {
            report.setMessage(strings.get("backup.notfound", name));
            report.setActionExitCode(ActionReport.ExitCode.FAILURE);
            return;
        }
        final BackupConfig backupConfig = backupConfigs.getBackupConfigByName(name);
        try {
            ConfigSupport.apply(new SingleConfigCode<BackupConfig>() {

                public Object run(BackupConfig param) throws
                        PropertyVetoException,
                        TransactionFailure {

                    param.setAutoBackupEnabled("true");

                    return param;
                }
            }, backupConfig);
        } catch(TransactionFailure e) {
            logger.info("TransactionFailure: " + e.toString());
            report.setMessage(strings.get("backup.enable.failure", name));
            report.setMessage(e.getLocalizedMessage());
            report.setActionExitCode(ActionReport.ExitCode.FAILURE);
            report.setFailureCause(e);
            return;
        }

        report.setActionExitCode(ActionReport.ExitCode.SUCCESS);
        return;
    }
}
