/*
 * Copyright (c) 2010, Oracle and/or its affiliates. All rights reserved.
 */

package com.sun.glassfish.dasrecovery;


import org.glassfish.api.I18n;
import org.glassfish.api.ActionReport;
import org.glassfish.api.ActionReport.ExitCode;
import org.glassfish.api.admin.*;
import org.glassfish.api.Param;

import com.sun.enterprise.config.serverbeans.Config;
import com.sun.enterprise.universal.i18n.LocalStringsImpl;
import com.sun.enterprise.util.StringUtils;
import com.sun.enterprise.admin.util.RowFormatter;

import org.jvnet.hk2.annotations.Service;
import org.jvnet.hk2.annotations.Inject;
import org.jvnet.hk2.annotations.Scoped;
import org.jvnet.hk2.component.PerLookup;
import org.jvnet.hk2.component.Habitat;
import org.jvnet.hk2.config.ConfigSupport;
import org.jvnet.hk2.config.SingleConfigCode;
import org.jvnet.hk2.config.TransactionFailure;

import java.util.Date;


/**
 * Remote asadmin command: list-backup-configs
 *
 * Purpose: Allows the invoker to list previous created backup-configs.
 *
 *
 * Implementation: 
 *
 *   If an operand is not given in normal mode:
 *       1) If no configs exist display nothing, return 0.
 *       2) If configs exist display backup config names, return 0.
 *   If an operand is not given in long mode:
 *       1) If no configs exist display nothing, return 0.
 *       2) If configs exist display backup config details, return 0.
 *   If an operand is given in normal mode:
 *       1) If no configs exist generate an error, return 1.
 *       2) If the named backup config is not in the list generate an 
 *            error, return 1
 *       3) If the named backup config exists display the name, return 0
 *   If an operand is given in long mode:
 *       1) If no configs exist generate an error, return 1.
 *       2) If the named backup config is not in the list generate an 
 *            error, return 1
 *       3) If the named backup config exists display the backup config 
 *            details, return 0
 *
 * @author Chris Kasso
 *
 */

@Service(name="list-backup-configs")
@ExecuteOn(RuntimeType.DAS)  // Only appropriate for the DAS
@Scoped(PerLookup.class)
@CommandLock(CommandLock.LockType.NONE)
@I18n("backup.list.command")
public class ListBackupConfigs implements AdminCommand {

    private static final String EOL = "\n";
    private static final String TAB = "\t";
    private final StringBuilder message = new StringBuilder();

    @Inject(name=ServerEnvironment.DEFAULT_INSTANCE_NAME)
    Config config;

    @Inject
    private Habitat habitat;

    @I18n("backup.list.name")
    @Param(primary = true, optional = true)
    private String name;

    @I18n("backup.list.verbose")
    @Param(name = "long", shortName="l", optional = true, defaultValue = "false")
    private boolean verbose;

    private static final LocalStringsImpl strings =
            new LocalStringsImpl(ListBackupConfigs.class);

    private final String NOTHING_TO_LIST=strings.get("nothing.to.list");


    private final String NAME = strings.get("backup.list.namebc");
    private final String BACKUP_DIR = strings.get("backup.list.backupdir");
    private final String SCHEDULE = strings.get("backup.list.schedule");
    private final String RECYCLE_LIMIT = strings.get("backup.list.recyclelimit");

    private final String CONFIG_ONLY = strings.get("backup.list.configonly");
    private final String AUTOBACKUP_ENABLED = strings.get("backup.list.autobackupenabled");
    private final String ACTIVEBACKUP_ENABLED = strings.get("backup.list.activebackupenabled");
    private final String LAST_BACKUP_SUCCESS = strings.get("backup.list.lastsuccessfulbackup");
    private final String LAST_BACKUP_ATTEMPT = strings.get("backup.list.lastattemptbackup");
    public void execute(AdminCommandContext context) {

        ActionReport report = context.getActionReport();

        BackupConfigs backupConfigs =
                config.getExtensionByType(BackupConfigs.class);

        /**
         * Check if any backupConfigs exist in this Config.
         */
        if (backupConfigs == null) {
            // If the user provided a name then it's an error if it is missing.
            if (StringUtils.ok(name)) {
                report.setMessage(strings.get("backup.list.notfound", name));
                report.setActionExitCode(ActionReport.ExitCode.FAILURE);
                return;
            }

            // No backupConfigs exist so just return.
            report.setActionExitCode(ActionReport.ExitCode.SUCCESS);
            return;
        }

        /**
         * List a specific BackupConfig.
         */
        if (StringUtils.ok(name)) {
            BackupConfig backupConfig =
                    backupConfigs.getBackupConfigByName(name);

            // The named backupConfig can not be found.
            if (backupConfig == null) {
                report.setMessage(
                        strings.get("backup.list.notfound", name));
                report.setActionExitCode(ActionReport.ExitCode.FAILURE);
                return;
            }

            // If not verbose just display the name (just like /bin/ls)
            if (!verbose)  {
                report.setMessage(name);
                report.setActionExitCode(ActionReport.ExitCode.SUCCESS);
                return;
            }

            StringBuilder sb = new StringBuilder();
            // Verbose case.  Fetch and format the details
            sb.append(generateBackupDetails(backupConfig));

            Schedules schedules = config.getExtensionByType(Schedules.class);
            if (schedules != null) {
                Schedule sched = schedules.getScheduleByName(
                    backupConfig.getSchedule());
                // The named schedule can not be found.
                if (sched != null) {
                    // If we are listing only a single config then we will also
                    // provide details about the schedule.
                    ScheduleFormatter sf = new ScheduleFormatter();
                    sf.formatSchedule(sched);
                    sb.append(EOL + strings.get("backup.list.scheddetails") +
                              EOL);
                    sb.append(sf.toString());
                } 
            }

            report.setMessage(sb.toString());
            report.setActionExitCode(ActionReport.ExitCode.SUCCESS);
            return;
        }
        if (backupConfigs.getBackupConfigs().isEmpty())   {

            report.setMessage(NOTHING_TO_LIST);
            report.setActionExitCode(ActionReport.ExitCode.SUCCESS);
            return;
        }

        /**
         * List all backupConfigs
         */
        for (BackupConfig backupConfig : backupConfigs.getBackupConfigs())  {
                if (verbose) {
                    // Verbose case.  Fetch and format the details
                    message.append(generateBackupDetails(backupConfig)+EOL) ;
                } else {
                    message.append(backupConfig.getName() +EOL);
                }

        }

        if (message.length() != 0)
            report.setMessage(message.substring(0,message.length()-1));
        report.setActionExitCode(ActionReport.ExitCode.SUCCESS);
        return;
    }


    private String generateBackupDetails(BackupConfig backupConfig) {

        String[] headings = new String[]{
                NAME,
                AUTOBACKUP_ENABLED, 
                SCHEDULE,
                RECYCLE_LIMIT,
                CONFIG_ONLY,
                ACTIVEBACKUP_ENABLED,
                BACKUP_DIR,
                LAST_BACKUP_ATTEMPT,
                LAST_BACKUP_SUCCESS
        };
        RowFormatter rf = new RowFormatter(headings);
        return rf.addColumn(new Object[]{
                backupConfig.getName(),
                backupConfig.getAutoBackupEnabled(),
                backupConfig.getSchedule(),
                backupConfig.getRecycleLimit() ,
                backupConfig.getConfigOnly(),
                backupConfig.getActiveBackupEnabled(),
                backupConfig.getBackupDir(),
                backupConfig.getTimeOfLastBackupAttempt()!=null?new Date(new Long(backupConfig.getTimeOfLastBackupAttempt())).toString():"",
                backupConfig.getTimeOfLastBackupSuccess()!=null?new Date(new Long(backupConfig.getTimeOfLastBackupSuccess())).toString():""


        });

    }

    
}
