/*
 * Copyright (c) 2010, Oracle and/or its affiliates. All rights reserved.
 */

package com.sun.glassfish.dasrecovery;


import org.glassfish.api.I18n;
import org.glassfish.api.ActionReport;
import org.glassfish.api.ActionReport.ExitCode;
import org.glassfish.api.admin.*;
import org.glassfish.api.Param;

import com.sun.enterprise.config.serverbeans.Config;
import com.sun.enterprise.universal.i18n.LocalStringsImpl;
import com.sun.enterprise.util.StringUtils;

import org.jvnet.hk2.annotations.Service;
import org.jvnet.hk2.annotations.Inject;
import org.jvnet.hk2.annotations.Scoped;
import org.jvnet.hk2.component.PerLookup;
import org.jvnet.hk2.component.Habitat;
import org.jvnet.hk2.config.ConfigSupport;
import org.jvnet.hk2.config.SingleConfigCode;
import org.jvnet.hk2.config.TransactionFailure;


/**
 * Remote asadmin command: list-schedules
 *
 * Purpose: Allows the invoker to list previous created schedules.
 *
 *
 * Implementation: 
 *
 *   If an operand is not given in normal mode:
 *       1) If no schedules exist display nothing, return 0.
 *       2) If schedules exist display schedule names, return 0.
 *   If an operand is not given in long mode:
 *       1) If no schedules exist display nothing, return 0.
 *       2) If schedules exist display schedule details, return 0.
 *   If an operand is given in normal mode:
 *       1) If no schedules exist generate an error, return 1.
 *       2) If the named schedule is not in the list generate an error, return 1
 *       3) If the named schedule exists display the name, return 0
 *   If an operand is given in long mode:
 *       1) If no schedules exist generate an error, return 1.
 *       2) If the named schedule is not in the list generate an error, return 1
 *       3) If the named schedule exists display the schedule details, return 0
 *
 * @author Chris Kasso
 *
 */

@Service(name="list-schedules")
@ExecuteOn(RuntimeType.DAS)  // Only appropriate for the DAS
@Scoped(PerLookup.class)
@CommandLock(CommandLock.LockType.NONE)
@I18n("schedule.list.command")
public class ListSchedules implements AdminCommand {

    private static final String EOL = "\n";
    private static final String TAB = "\t";
    private final StringBuilder message = new StringBuilder();

    @Inject(name=ServerEnvironment.DEFAULT_INSTANCE_NAME)
    Config config;

    @Inject
    private Habitat habitat;

    @I18n("schedule.list.name")
    @Param(primary = true, optional = true)
    private String name;

    @I18n("schedule.list.verbose")
    @Param(name = "long", shortName="l", optional = true, defaultValue = "false")
    private boolean verbose;

    private static final LocalStringsImpl strings =
            new LocalStringsImpl(ListSchedules.class);

    public void execute(AdminCommandContext context) {

        ActionReport report = context.getActionReport();

        Schedules schedules = config.getExtensionByType(Schedules.class);

        /**
         * Check if any schedules exist in this Config.
         */
        if (schedules == null) {
            // If the user provided a name then it's an error if it is missing.
            if (StringUtils.ok(name)) {
                report.setMessage(strings.get("schedule.list.notfound", name));
                report.setActionExitCode(ActionReport.ExitCode.FAILURE);
                return;
            }

            // No schedules exist so just return.
            report.setActionExitCode(ActionReport.ExitCode.SUCCESS);
            return;
        }

        ScheduleFormatter sf = new ScheduleFormatter();

        /**
        * List a specific schedule.
        */
        if (StringUtils.ok(name)) {
            Schedule sched = schedules.getScheduleByName(name);

            // The named schedule can not be found.
            if (sched == null) {
                report.setMessage(
                        strings.get("schedule.list.notfound", name));
                report.setActionExitCode(ActionReport.ExitCode.FAILURE);
                return;
            }

            // If not verbose just display the name (just like /bin/ls)
            if (!verbose)  {
                report.setMessage(name);
                report.setActionExitCode(ActionReport.ExitCode.SUCCESS);
                return;
            }

            // Verbose case.  Fetch and format the details
            sf.formatSchedule(sched);
            message.append(sf.toString());

            report.setMessage(message.toString());
            report.setActionExitCode(ActionReport.ExitCode.SUCCESS);
            return;
        }

        /**
         * List all schedules
         */
        boolean first=true;
        for (Schedule sched : schedules.getSchedules()) {
            if (verbose) {
                // Verbose case.  Fetch and format the details
                sf.formatSchedule(sched);
            } else {
                if(first)
                    first = false;
                else
                    message.append(EOL);
                message.append(sched.getName());
            }
        }
        if (verbose) {
            message.append(sf.toString());
        }
        report.setMessage(message.toString());
        report.setActionExitCode(ActionReport.ExitCode.SUCCESS);
        return;
    }
}
