/*
 * Copyright (c) 2010, Oracle and/or its affiliates. All rights reserved.
 */

package com.sun.glassfish.dasrecovery;

import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.net.URL;
import java.io.IOException;

import org.glassfish.api.admin.config.ConfigParser;
import org.glassfish.api.ActionReport;
import org.glassfish.api.ActionReport.ExitCode;

import org.glassfish.api.I18n;
import org.glassfish.api.Param;
import org.glassfish.api.admin.*;

import org.glassfish.cluster.ssh.connect.NodeRunner;

import com.sun.enterprise.config.serverbeans.Node;
import com.sun.enterprise.config.serverbeans.Nodes;
import com.sun.enterprise.config.serverbeans.Config;
import com.sun.enterprise.config.serverbeans.Configs;
import com.sun.enterprise.util.SystemPropertyConstants;
import com.sun.enterprise.universal.process.LocalAdminCommand;
import com.sun.enterprise.universal.process.ProcessManagerException;
import com.sun.enterprise.universal.i18n.LocalStringsImpl;

import com.sun.enterprise.util.StringUtils;

import org.jvnet.hk2.annotations.Service;
import org.jvnet.hk2.annotations.Inject;
import org.jvnet.hk2.annotations.Scoped;
import org.jvnet.hk2.config.ConfiguredBy;
import org.jvnet.hk2.component.PostConstruct;
import org.jvnet.hk2.component.PerLookup;
import org.jvnet.hk2.component.Habitat;

/**
 * Remote asadmin command: update-admin-server-coordinates
 * 
 * Purpose: Allows the invoker to update the coordinates (hostname, port)
 * of the DAS for all known nodes.
 * 
 * Implementation: This remote command iterates through the list of nodes
 * for the domain.  For each node it invokes the local asadmin command
 * update-admin-server-local-coordinates.
 * This implementation depends on SSH to be enabled between the DAS and the
 * nodes.
 * 
 * If a node name is provided as an operand then only that node is updated.
 * 
 * Config nodes are ignored unless a config node is passed as an operand in
 * which case that is an error.
 *
 * If --adminhost is not provivded we will use the hostname of the DAS.
 * If --adminport is not provivded we will use the port of the DAS.
 *
 * @author Chris Kasso
 *
 */

/**
 * TODO: 
 * 
 * - If we determine the new hostname the hostname we compute is not fully
 *   qualified.
 *
 */

@Service(name = "update-admin-server-coordinates")
@ExecuteOn(RuntimeType.DAS)  // Only appropriate for the DAS
@Scoped(PerLookup.class)
@CommandLock(CommandLock.LockType.NONE)
@I18n("update.coords.command")
public class UpdateServerCoordsCommand implements AdminCommand, PostConstruct {

    @I18n("update.coords.adminhost")
    @Param(name = "adminhost", optional = true)
    private String adminHost;  //To be set in das.properties:das.host

    @Param(name = "adminport", optional = true)
    private String adminPort;  //To be set in das.properties:das.port

    @I18n("update.coords.nodename")
    @Param(optional = true, primary = true)
    private String nodeName; 

    @I18n("update.coords.verbose")
    @Param(name = "long", shortName="l", optional = true, defaultValue = "false")
    private boolean verbose; 

    @Inject
    Logger logger;

    @Inject
    Habitat habitat;

    @Inject 
    Node[] nodeList;

    @Inject 
    private Nodes nodes;

    private static enum NodeValidation { NOERROR, NOEXIST, NOHOST, NOHOME };
    private static final String NL = System.getProperty("line.separator");

    private static final LocalStringsImpl strings =
            new LocalStringsImpl(UpdateServerCoordsCommand.class);

    private UpdateServerCoordsHelper uscHelper;

    @Override
    public void postConstruct() {
        uscHelper = new UpdateServerCoordsHelper(habitat);
    }


    public void execute(AdminCommandContext context) {
        ActionReport report = context.getActionReport();
        String dasHost = System.getProperty(
                                   SystemPropertyConstants.HOST_NAME_PROPERTY);

        if (!StringUtils.ok(adminHost)) { 
            logger.info("--adminhost not provided. Using DAS host: " + dasHost);
            //XXX: The dasHost is not a fully qualified hostname!!!
            adminHost = dasHost;
        } else {
            logger.fine("--adminhost is: " + adminHost);
        }

        if (!StringUtils.ok(adminPort)) { 
            int dasPort = uscHelper.getAdminPort(
                                   SystemPropertyConstants.DAS_SERVER_NAME);
            logger.info("--adminport not provided. Using DAS port: " +
                        Integer.toString(dasPort));
            adminPort = Integer.toString(dasPort);
        } else {
            logger.fine("--adminport is: " + adminPort);
        }


        /* Handle the case where we were provided with the node name to be
         * updated.
         */
        if (StringUtils.ok(nodeName)) {
            if (validateNode(report, nodeName) == NodeValidation.NOERROR) {
                updateSingleNode(report, nodeName, adminHost, adminPort);
                if (report.getActionExitCode() != ExitCode.SUCCESS) {
                    String errMsg = strings.get("update.coords.failed",
                                                nodeName);
                    String msg = report.getMessage();
                    report.setMessage(errMsg + "\n" + msg);
                    logger.severe(errMsg);
                } else {
                    if (verbose) {
                        report.setMessage(strings.get(
                            "update.coords.succeeded", nodeName));
                    }
                }
            } else { 
                String errMsg = strings.get("update.coords.failed", nodeName);
                String msg = report.getMessage();
                report.setMessage(errMsg + "\n" + msg);
                logger.severe(errMsg);
                return;
            }
            return;
        }


        /* Save command output to return in the ActionReport */
        StringBuilder output = new StringBuilder();
        StringBuilder errorOutput = new StringBuilder();
        StringBuilder failedNodeNames = new StringBuilder();
        StringBuilder succeededNodeNames = new StringBuilder();
        boolean failureOccurred = false;


        /* Handle the case where the node was not provided as an operand
         * to the command.  We update all nodes.  We skip those nodes
         * which are config-nodes.
         */
        for (Node node : nodeList) {
            nodeName = node.getName();

            if (!node.nodeInUse()) {
                // We ignore nodes that have no instances.
                logger.info("update-admin-server-coordinates: " +
                            "Ignoring node with no server instances: " +
                            nodeName);
                continue;
            }

            if (node.isDefaultLocalNode()) {
                // We ignore "localhost" nodes.
                // Can't use node.isLocal() here because we should update
                // ssh nodes created on the host that are not the "localhost"
                // node.
                logger.info("update-admin-server-coordinates: " +
                            "Ignoring local node: " +
                            nodeName);
                continue;
            }

            /* If the node is not valid it is not an error.  All the nodes
             * should be valid nodes.  Nodes that are missing information
             * are assumed to be config nodes.
             */
            if (validateNode(report, nodeName) == NodeValidation.NOERROR) {
                updateSingleNode(report, nodeName, adminHost, adminPort);
                if (report.getActionExitCode() != ExitCode.SUCCESS) {
                    // The node could not be updated.   
                    failureOccurred = true;
                    failedNodeNames.append(nodeName).append(" ");
                    errorOutput.append(NL).append(report.getMessage()).append(NL);
                }
            }
            if (report.getActionExitCode() == ExitCode.SUCCESS) { 
                // The command worked for this node.
                succeededNodeNames.append(nodeName).append(" ");
            }
        }

        report.setActionExitCode(ExitCode.SUCCESS);

        // Display a summary of updated nodes if a failure occurred or if
        // we are in verbose mode.
        if (succeededNodeNames.length() > 0 && (verbose || failureOccurred)) {
            output.append(strings.get("update.coords.succeeded",
                          succeededNodeNames));
            if (failureOccurred)
                output.append(NL);
        }

        if (failureOccurred) {
            output.append(strings.get("update.coords.faileds",
                                failedNodeNames));
            if (succeededNodeNames.length() > 0) {
                // At least one node was updated. Warning.
                report.setActionExitCode(ExitCode.WARNING);
            } else {
                // No nodes updated. Failure
                report.setActionExitCode(ExitCode.FAILURE);
            }
            logger.warning(output.toString());
            logger.warning(errorOutput.toString());
            report.setMessage(output.toString() + NL + errorOutput.toString());
        } else {
            logger.info(output.toString());
            report.setMessage(output.toString());
        }

        return;
    }


    /* Validate the node exists and is usable (could be a config-node) */
    private NodeValidation validateNode(ActionReport report, String nodeName) {
 
        String msg;
        Node node = nodes.getNode(nodeName);

        if (node == null) {
            msg = strings.get("update.coords.nodenoname", nodeName);
            logger.info(msg);
            report.setMessage(msg);
            report.setActionExitCode(ExitCode.FAILURE);
            return NodeValidation.NOEXIST;
        }

        if (!StringUtils.ok(node.getNodeHost())) {
            msg = strings.get("update.coords.nodenohost", nodeName);
            logger.info(msg);
            report.setMessage(msg);
            report.setActionExitCode(ExitCode.FAILURE);
            return NodeValidation.NOHOST;
        }

        if (!StringUtils.ok(node.getInstallDir())) {
            msg = strings.get("update.coords.nodenoinstalldir", nodeName);
            logger.info(msg);
            report.setMessage(msg);
            report.setActionExitCode(ExitCode.FAILURE);
            return NodeValidation.NOHOME;
        }
        return NodeValidation.NOERROR;
    }

    
    /* This updates a single node with the new host and port.  The node
     * can be local or remote.
     */
    private void updateSingleNode(ActionReport report, String nodeName,
                                  String adminHost, String adminPort) {
            
        Node node = nodes.getNode(nodeName);
        String nodeHost = node.getNodeHost();
        NodeRunner nr = new NodeRunner(habitat, logger);
        ArrayList<String> command = new ArrayList<String>();
        String msg = null;
        boolean failure = true;

        command.add("update-admin-server-local-coordinates");
        command.add("--adminhost");
        command.add(adminHost);
        command.add("--adminport");
        command.add(adminPort);
        command.add(nodeName);

        try {
            StringBuilder output = new StringBuilder();
            int status = nr.runAdminCommandOnNode(node, output, true, command);

            if (status != 0) {
                // Command ran, but didn't succeed.
                msg = strings.get("update.coords.localnode.failed",
                                         nodeName, nodeHost, 
                                         output.toString().trim(), 
                                         nr.getLastCommandRun());
                logger.warning(msg);
            } else {
                failure = false;
                logger.info(output.toString().trim());
            }
        } catch (SSHCommandExecutionException e) {
            msg = strings.get("update.coords.ssh.bad.connect",
                              nodeName, nodeHost, e.getMessage());
            // Log some extra info
            String msg1 = strings.get("update.coords.failed.ssh.details",
                nodeName, nodeHost, e.getCommandRun(), e.getMessage(),
                e.getSSHSettings());
            logger.warning("update-admin-server-coordinates:" + msg1);
        } catch (ProcessManagerException e) {
            msg = strings.get("update.coords.failed.local.details",
                e.getMessage(), nr.getLastCommandRun());
            logger.warning("update-admin-server-coordinates:" + msg);
            // User message doesn't have command that was run
            msg = strings.get("update.coords.failed.local.exception",
                              e.getMessage());
        } catch (UnsupportedOperationException e) {
            msg = strings.get("update.coords.failed.node.not.ssh", nodeName,
                              nodeHost);
            logger.warning("update-admin-server-coordinates:" + msg);
        } catch (IllegalArgumentException e) {
            msg = e.getMessage();
            logger.warning("update-admin-server-coordinates:" + msg);
        }

        if (failure) {
            report.setActionExitCode(ExitCode.FAILURE);
            report.setMessage(msg);
        } else {
            report.setActionExitCode(ExitCode.SUCCESS);
        }
    } 
}
