/*
 * Copyright (c) 2010, Oracle and/or its affiliates. All rights reserved.
 */

package com.sun.glassfish.quiesce;

import java.util.Date;
import java.text.SimpleDateFormat;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.glassfish.api.I18n;
import org.glassfish.api.admin.config.ConfigExtension;
import org.glassfish.api.ActionReport;
import org.glassfish.api.ActionReport.ExitCode;
import org.glassfish.api.admin.AdminCommandLock.SuspendStatus;
import org.glassfish.api.admin.*;
import org.glassfish.api.Param;

import com.sun.enterprise.universal.i18n.LocalStringsImpl;
import com.sun.enterprise.util.StringUtils;

import org.jvnet.hk2.annotations.Service;
import org.jvnet.hk2.annotations.Inject;
import org.jvnet.hk2.annotations.Scoped;
import org.jvnet.hk2.component.PerLookup;

/**
 * Remote asadmin command: suspend-domain
 * 
 * Purpose: Allows the invoker to quiesce the DAS.
 * 
 * @author Chris Kasso
 *
 */

@Service(name="suspend-domain")
@ExecuteOn(RuntimeType.DAS)  // Only appropriate for the DAS
@Scoped(PerLookup.class)
@CommandLock(CommandLock.LockType.NONE)
@I18n("quiesce.command.suspend")
public class SuspendDomain implements AdminCommand {

    private static final int TIMEOUT = 30; 

    @Inject
    Logger logger;

    @Inject
    private AdminCommandLock adminCommandLock;

    @Param(name = "timeout", optional = true)
    private String timeout_in_seconds;

    @Param(name = "message", optional = true)
    private String message;

    @Param(name = "_test", optional = true)
    private String test;


    private static final LocalStringsImpl strings =
            new LocalStringsImpl(SuspendDomain.class);

    public void execute(AdminCommandContext context) {

        ActionReport report = context.getActionReport();
        AdminCommandLock.SuspendStatus suspendStatus;
        long timeout = TIMEOUT;

        if (timeout_in_seconds != null) {
            try {
                timeout = Long.parseLong(timeout_in_seconds);
                if (timeout < 0) {
                    timeoutError(report, timeout_in_seconds);
                    return;
                }
            } catch (NumberFormatException e) {
                timeoutError(report, timeout_in_seconds);
                return;
            }
        } else {
            boolean badTimeOutValue = false;
            String timeout_s = System.getProperty(
                "com.sun.aas.commandLockTimeOut", "30");

            try {
                timeout = Integer.parseInt(timeout_s);
                if (timeout < 0)
                    badTimeOutValue = true;
            } catch (NumberFormatException e) {
                badTimeOutValue = true;
            }
            if (badTimeOutValue) {
                //XXX: Deal with logger injection attack.
                logger.log(Level.INFO,
                           "Bad value com.sun.aas.commandLockTimeOut: "
                           + timeout_s + ". Using 30 seconds.");
                timeout = TIMEOUT;
            }
        }

        /* 
         * If the hidden test flag was passed then we only test whether the 
         * domain is suspended.
         */
        if (test != null) {
            if (adminCommandLock.isSuspended()) {
                // Do not localize these strings.
                report.setMessage("SUSPENDED=TRUE");
            } else {
                report.setMessage("SUSPENDED=FALSE");
            }
            report.setActionExitCode(ActionReport.ExitCode.SUCCESS);
            return;
        }

        //XXX: Figure out how to retrieve the admin user id.
        String user = new String("asadmin");

        /*
         * Ask the DAS to suspend executing commands.  This method does
         * not return until the DAS is in a suspended state (using an 
         * EXCLUSIVE lock), times out trying or an error occurs.
         */
        if (message != null)
            suspendStatus = adminCommandLock.suspendCommands(timeout, user,
                                                             message);
        else
            suspendStatus = adminCommandLock.suspendCommands(timeout, user);

        /*
         * Check the status to determine if the lock was successful.
         */
        if (suspendStatus == SuspendStatus.ILLEGALSTATE) {
            String lockMsg = adminCommandLock.getLockMessage();
            String msg;

            msg = strings.get("quiesce.alreadysuspended", 
                    formatSuspendDate(
                            adminCommandLock.getLockTimeOfAcquisition()));

            if (lockMsg != null && lockMsg != "") {
                msg += " "
                    + strings.get("quiesce.reason")
                    + " " + lockMsg;
            }

            report.setMessage(msg);
            report.setActionExitCode(ActionReport.ExitCode.FAILURE);
            return;
        } else if (suspendStatus == AdminCommandLock.SuspendStatus.TIMEOUT) {
            report.setMessage(strings.get("quiesce.suspendtimeout",
                    formatSuspendDate(
                            adminCommandLock.getLockTimeOfAcquisition())));
            report.setActionExitCode(ActionReport.ExitCode.FAILURE);
            return;
        } else if (suspendStatus == AdminCommandLock.SuspendStatus.ERROR) {
            report.setMessage(strings.get("quiesce.error"));
            report.setActionExitCode(ActionReport.ExitCode.FAILURE);
            return;
        }

        report.setMessage(strings.get("quiesce.suspendsuccessful",
            formatSuspendDate(adminCommandLock.getLockTimeOfAcquisition())));
        if (message != null && message != "") 
            // XXX: Logger injection threat....
            logger.log(Level.INFO, strings.get("quiesce.suspendsuccessfulmsg",
                formatSuspendDate(adminCommandLock.getLockTimeOfAcquisition()),
                message));
        else
            logger.log(Level.INFO, strings.get("quiesce.suspendsuccessful",
                formatSuspendDate(adminCommandLock.getLockTimeOfAcquisition())));
        report.setActionExitCode(ActionReport.ExitCode.SUCCESS);
        return;
    }

    private String formatSuspendDate(Date lockTime) {
        if (lockTime != null) {
            String DATE_FORMAT = "EEE, dd MMM yyyy HH:mm:ss z";
            SimpleDateFormat sdf = new SimpleDateFormat(DATE_FORMAT);
            return sdf.format(lockTime);
        } else {
            return new String(strings.get("quiesce.dateunavailable"));
        }
    }

    private void timeoutError(ActionReport report, String timeout) {
        report.setMessage(strings.get("quiesce.badtimeout", timeout));
        report.setActionExitCode(ActionReport.ExitCode.FAILURE);
        return;
    }
}
